/**
 * @license
 * Copyright 2019 Google LLC
 * SPDX-License-Identifier: BSD-3-Clause
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { LitElement, html, css } from 'lit';
import { customElement, property, queryAsync } from 'lit/decorators.js';
/**
 * An example element.
 *
 * @fires count-changed - Indicates when the count changes
 * @slot - This element has a slot
 * @csspart button - The button
 */
let MercuryDialog = class MercuryDialog extends LitElement {
    constructor() {
        super(...arguments);
        /**
         * Whether the dialog should be open or not.
         */
        this.open = false;
        /**
         * Whether the dialog should be treated as a modal with a backdrop.
         */
        this.modal = false;
        /**
         * Whether the dialog should be moveable.
         */
        this.moveable = false;
        /**
         * The location in which the dialog should be docked.
         */
        this.dock = 'none';
        this.closeButton = true;
        this.backdropOpacity = 1;
        /**
         * The title shown in the dialog header when open.
         */
        this.title = '';
        this._dragStartX = 0;
        this._dragStartY = 0;
        this._offsetTop = 0;
        this._offsetLeft = 0;
        this._onDragMouseUp = () => {
            document.removeEventListener('mouseup', this._onDragMouseUp);
            document.removeEventListener('mousemove', this._onMouseMove);
        };
        this._onMouseMove = (event) => {
            this._dialog.style.left = `${Math.min(Math.max(0, this._offsetLeft + event.clientX - this._dragStartX), window.innerWidth - this._dialog.offsetWidth)}px`;
            this._dialog.style.top = `${this._offsetTop + event.clientY - this._dragStartY}px`;
            this._dialog.style.right = 'auto';
            this._dialog.style.bottom = 'auto';
        };
    }
    render() {
        return html `
      <dialog
        id="dialog"
        part="dialog"
        data-dock=${this.dock}
        ?open=${this.open}
      >
        ${this.title || this.closeButton
            ? html `<header>
              ${this.title ? html `<h2>${this.title}</h2>` : html ``}
              ${this.moveable
                ? html `<button
                    @mousedown=${this._onDragMouseDown}
                    part="drag-button"
                    id="dragButton"
                  >
                    Drag
                  </button>`
                : html ``}
              ${this.dock === 'none'
                ? html `<button
                    @click=${() => this._setDock('right')}
                    part="dock-button"
                    id="dockButton"
                  >
                    Dock
                  </button>`
                : html `<button
                    @click=${() => this._setDock('none')}
                    part="undock-button"
                    id="undockButton"
                  >
                    Undock
                  </button>`}
              <form method="dialog">
                <button
                  @click=${this._onCloseClick}
                  part="close-button"
                  id="closeButton"
                >
                  Close
                </button>
              </form>
            </header>`
            : html ``}

        <slot></slot>
      </dialog>
    `;
    }
    /**
     * showModal
     * Shows the dialog as a modal with a background overlay.
     */
    async showModal() {
        this.open = true;
        this.modal = true;
        (await this._dialog).showModal();
    }
    /**
     * show
     * Shows the dialog.
     */
    async show() {
        this.open = true;
        this.modal = false;
        (await this._dialog).show();
    }
    /**
     * close
     * Closes the dialog.
     */
    close() {
        this.open = false;
        this._dialog.close();
    }
    _onCloseClick() {
        this.open = false;
    }
    _onDragMouseDown(event) {
        this._dragStartX = event.clientX;
        this._dragStartY = event.clientY;
        this._offsetLeft = this._dialog.offsetLeft;
        this._offsetTop = this._dialog.offsetTop;
        document.addEventListener('mouseup', this._onDragMouseUp);
        document.addEventListener('mousemove', this._onMouseMove);
    }
    _setDock(direction) {
        this.dock = direction;
        switch (direction) {
            case 'right':
                Object.assign(this._dialog.style, {
                    inset: '0 0 0 auto',
                });
                break;
            case 'none':
                Object.assign(this._dialog.style, {
                    inset: 'auto',
                });
                break;
            default:
                break;
        }
    }
};
MercuryDialog.styles = css `
    :host {
      display: block;
      font-family: var(--me-font-family, sans-serif);
    }

    dialog {
      border-style: var(--me-border-style, solid);
      border-width: var(--me-border-width, 1px);
      border-color: var(--me-border-color, #e5e5e5);
      padding: 0;
    }

    dialog[data-dock='right'] {
      height: auto;
      max-height: none;
      width: 400px;
      left: auto;
      right: 0;
      position: fixed;
      top: 0;
      bottom: 0;
      border-right-width: 0;
      border-top-width: 0;
      border-bottom-width: 0;
    }

    dialog[data-dock='left'] {
      height: auto;
      max-height: none;
      width: 400px;
      left: 0;
      right: auto;
      position: fixed;
      top: 0;
      bottom: 0;
    }

    dialog::backdrop {
      background-color: black;
      opacity: 0.4;
    }

    #dragButton {
      cursor: move;
    }

    header {
      display: flex;
      justify-content: flex-end;
      padding: var(--me-space-inset, 8px);
      border-bottom-style: var(--me-border-style, solid);
      border-bottom-width: var(--me-border-width, 1px);
      border-bottom-color: var(--me-border-color, #e5e5e5);
    }

    h2 {
      font-size: var(--me-label-font-size, 16px);
      line-height: var(--me-label-line-height, 1.1);
      margin-inline-end: auto;
    }

    form[method='dialog'] {
      align-items: stretch;
      display: flex;
    }
  `;
__decorate([
    property({ type: Boolean, reflect: true })
], MercuryDialog.prototype, "open", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], MercuryDialog.prototype, "modal", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], MercuryDialog.prototype, "moveable", void 0);
__decorate([
    property({ type: String, reflect: true })
], MercuryDialog.prototype, "dock", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], MercuryDialog.prototype, "closeButton", void 0);
__decorate([
    property({ type: Number, reflect: true })
], MercuryDialog.prototype, "backdropOpacity", void 0);
__decorate([
    property({ type: String, reflect: true })
], MercuryDialog.prototype, "title", void 0);
__decorate([
    queryAsync('#dialog')
], MercuryDialog.prototype, "_dialog", void 0);
MercuryDialog = __decorate([
    customElement('mercury-dialog')
], MercuryDialog);
export { MercuryDialog };
//# sourceMappingURL=mercury-dialog.js.map